!!
!! This module computes surface conditions
!! - albedo
!! - roughness
!! - emissivity
!!
!! @author Marie-Alice Foujols and Jan Polcher
!! @Version : $Revision: 1.15.2.2 $, $Date: 2003/11/18 16:23:30 $
!! 
!! $Header: /home/ssipsl/CVSREP/ORCHIDEE/src_sechiba/condveg.f90,v 1.15.2.2 2003/11/18 16:23:30 ssipsl Exp $
!! 
MODULE condveg
  !
  USE ioipsl
  !
  ! modules used :
  USE constantes
  USE constantes_veg

  IMPLICIT NONE

  ! public routines :
  ! condveg_main only
  PRIVATE
  PUBLIC :: condveg_main,condveg_clear 

  !
  ! variables used inside condveg module : declaration and initialisation
  !
  LOGICAL, SAVE                     :: l_first_condveg=.TRUE.           !! To keep first call's trace
  LOGICAL, SAVE                     :: z0cdrag_ave=.FALSE.              !! Chooses the method for the z0 average
  !
  REAL(stnd), SAVE                  :: fixed_snow_albedo                !! In case we wish a fxed snow albedo
  INTEGER(short), PARAMETER         :: ivis = 1                         !! index for visible albedo
  INTEGER(short), PARAMETER         :: inir = 2                         !! index for near infrared albedo
  LOGICAL, SAVE                     :: impaze                           !! Choice on the surface parameters
  !
  REAL(stnd), SAVE                  :: z0_scal                          !! Roughness used to initialize the scheme
  REAL(stnd), SAVE                  :: roughheight_scal                 !! Height to displace the surface 
                                                                        !! from the zero wind height.
  REAL(stnd), SAVE                  :: albedo_scal(2)                   !! Two albedos used to initialize the scheme
  REAL(stnd), SAVE                  :: emis_scal                        !! Surface emissivity  used to initialize the scheme
  !
  REAL(stnd), ALLOCATABLE, SAVE     :: soilalb_dry(:,:)                 !! albedo for the dry bare soil
  REAL(stnd), ALLOCATABLE, SAVE     :: soilalb_wet(:,:)                 !! albedo for the wet bare soil
  REAL(stnd), PARAMETER             :: z0_over_height = un/16.          !! to get z0 from height
  REAL(stnd), PARAMETER             :: height_displacement = 0.75       !! Magic number which relates the 
                                                                          !! height to the displacement height.
  !
  REAL(stnd), ALLOCATABLE, SAVE, DIMENSION (:)  :: albedo_snow      !! Snow albedo
  REAL(stnd), ALLOCATABLE, SAVE, DIMENSION (:)  :: albedo_glob      !! Mean albedo

CONTAINS

  !! 
  !! Main routine for *condveg* module
  !! - called only one time for initialisation
  !! - called every time step
  !! - called one more time at last time step for writing _restart_ file
  !!
  !! Algorithm:
  !! - call condveg_init for initialisation
  !! - call condveg_var_init for initialisation done every time step
  !! - call condveg_snow for computing the modification of the albedo induced by snow cover
  !!
  !!  @call condveg_init
  !!  @call condveg_var_init
  !!  @call condveg_snow
  !!
  !!
  SUBROUTINE condveg_main (kjit, kjpindex, dtradia, ldrestart_read, ldrestart_write, index,&
       & lalo, neighbours, resolution, veget, veget_max, frac_nobio, totfrac_nobio, &
       & zlev, snow, snow_age, snow_nobio, snow_nobio_age, &
       & hdry, height, deadleaf_cover, emis, albedo, z0, roughheight, soiltype, rest_id, hist_id)

    ! interface description:
    ! input scalar 
    INTEGER(long), INTENT(in)                       :: kjit             !! Time step number 
    INTEGER(long), INTENT(in)                       :: kjpindex         !! Domain size
    INTEGER(long),INTENT (in)                       :: rest_id          !! _Restart_ file identifier
    INTEGER(long),INTENT (in)                       :: hist_id          !! _History_ file identifier
    REAL(stnd), INTENT (in)                         :: dtradia          !! Time step in seconds
    LOGICAL, INTENT(in)                             :: ldrestart_read   !! Logical for restart file to read
    LOGICAL, INTENT(in)                             :: ldrestart_write  !! Logical for restart file to write
    ! input fields
    INTEGER(long),DIMENSION (kjpindex), INTENT (in) :: index            !! Indeces of the points on the map
    REAL(stnd),DIMENSION (kjpindex,2), INTENT (in)  :: lalo             !! Geographical coordinates
    INTEGER(long),DIMENSION (kjpindex,8), INTENT(in):: neighbours       !! neighoring grid points if land
    REAL(stnd), DIMENSION (kjpindex,2), INTENT(in)  :: resolution       !! size in x an y of the grid (m)
    REAL(stnd),DIMENSION (kjpindex,nvm), INTENT(in) :: veget            !! Fraction of vegetation types
    REAL(stnd),DIMENSION (kjpindex,nvm), INTENT(in) :: veget_max        !! Fraction of vegetation type
    REAL(stnd),DIMENSION (kjpindex,nnobio), INTENT(in) :: frac_nobio    !! Fraction of continental ice, lakes, ...
    REAL(stnd),DIMENSION (kjpindex), INTENT(in)     :: totfrac_nobio    !! total fraction of continental ice+lakes+...
    REAL(stnd),DIMENSION (kjpindex), INTENT (in)    :: zlev             !! Height of first layer
    REAL(stnd),DIMENSION (kjpindex), INTENT(in)     :: snow             !! Snow mass [Kg/m^2]
    REAL(stnd),DIMENSION (kjpindex), INTENT(in)     :: snow_age         !! Snow age
    REAL(stnd),DIMENSION (kjpindex,nnobio), INTENT(in) :: snow_nobio    !! Snow mass [Kg/m^2] on ice, lakes, ...
    REAL(stnd),DIMENSION (kjpindex), INTENT(in)     :: snow_nobio_age   !! Snow age on ice, lakes, ...
    REAL(stnd),DIMENSION (kjpindex), INTENT(in)     :: hdry             !! Dry soil heigth in meters
    REAL(stnd),DIMENSION (kjpindex,nvm), INTENT(in) :: height           !! Vegetation Height (m)
    REAL(stnd),DIMENSION (kjpindex), INTENT(in)     :: deadleaf_cover   !! Fraction of soil covered by dead leaves
    ! output scalar
    ! output fields
    REAL(stnd),DIMENSION (kjpindex), INTENT (out)   :: emis             !! Emissivity
    REAL(stnd),DIMENSION (kjpindex,2), INTENT (out) :: albedo           !! Albedo, vis(1) and nir(2)
    REAL(stnd),DIMENSION (kjpindex), INTENT (out)   :: z0               !! Roughness
    REAL(stnd),DIMENSION (kjpindex), INTENT (out)   :: roughheight      !! Effective height for roughness
    REAL(stnd),DIMENSION (kjpindex,nbsoil), INTENT (in) :: soiltype     !! fraction of soil types
    ! local
    CHARACTER*80                                    :: var_name         !! To store variables names for I/O
    !
    IF (l_first_condveg) THEN

        IF (long_print) WRITE (numout,*) ' l_first_condveg : call condveg_init '

        CALL condveg_init (kjit, ldrestart_read, kjpindex, index, veget, &
                           lalo, neighbours, resolution, rest_id)
        CALL condveg_var_init (ldrestart_read, kjpindex, veget, veget_max, frac_nobio, totfrac_nobio, &
             & hdry, zlev, height, deadleaf_cover, emis, albedo, z0, roughheight)
        CALL condveg_snow (ldrestart_read, kjpindex, veget, frac_nobio, totfrac_nobio, &
                           snow, snow_age, snow_nobio, snow_nobio_age, albedo, albedo_snow, albedo_glob)

        RETURN

    ENDIF

    CALL condveg_var_update (ldrestart_read, kjpindex, veget, veget_max, frac_nobio, totfrac_nobio, &
         & hdry, zlev, height, deadleaf_cover, emis, albedo, z0, roughheight)
    CALL condveg_snow (ldrestart_read, kjpindex, veget, frac_nobio, totfrac_nobio, &
                       snow, snow_age, snow_nobio, snow_nobio_age, albedo, albedo_snow, albedo_glob)

    IF (ldrestart_write) THEN
       !
       var_name = 'soilalbedo_dry'
       CALL restput (rest_id, var_name, kjpindex, 2, 1, kjit, soilalb_dry, 'scatter',  kjpindex, index)
       !
       var_name = 'soilalbedo_wet'
       CALL restput (rest_id, var_name, kjpindex, 2, 1, kjit, soilalb_wet, 'scatter',  kjpindex, index)
       !
       RETURN
       !
    ENDIF

    IF ( almaoutput ) THEN
       CALL histwrite(hist_id, 'Albedo', kjit, albedo_glob, kjpindex, index)
       CALL histwrite(hist_id, 'SAlbedo', kjit, albedo_snow, kjpindex, index)
    ENDIF

    IF (long_print) WRITE (numout,*)' condveg_main done '

  END SUBROUTINE condveg_main

  !! Algorithm:
  !! - dynamic allocation for local array 
  !!
  SUBROUTINE condveg_init  (kjit, ldrestart_read, kjpindex, index, veget, &
                            lalo, neighbours, resolution, rest_id)

    ! interface description
    ! input scalar 
    INTEGER(long), INTENT(in)                       :: kjit             !! Time step number 
    LOGICAL, INTENT(in)                             :: ldrestart_read   !! Logical for restart file to read
    INTEGER(long), INTENT(in)                       :: kjpindex         !! Domain size
    INTEGER(long),DIMENSION (kjpindex), INTENT (in) :: index            !! Indeces of the points on the map
    REAL(stnd),DIMENSION (kjpindex,nvm), INTENT (in):: veget            !! Vegetation distribution
    REAL(stnd),DIMENSION (kjpindex,2), INTENT (in)  :: lalo             !! Geographical coordinates
    INTEGER(long),DIMENSION (kjpindex,4), INTENT(in):: neighbours       !! neighoring grid points if land
    REAL(stnd), DIMENSION (kjpindex,2), INTENT(in)  :: resolution       !! size in x an y of the grid (m)
    INTEGER(long), INTENT(in)                       :: rest_id          !! Restart file identifier
    ! input fields
    ! output scalar
    ! output fields

    ! local declaration
    INTEGER(short)                                  :: ji
    INTEGER(short)                                  :: ier
    CHARACTER*80                                    :: var_name         !! To store variables names for I/O
    ! initialisation
    IF (l_first_condveg) THEN 
       !
       ! Get the fixed snow albedo if needed
       !
       !
       !Config Key  = CONDVEG_SNOWA
       !Config Desc = The snow albedo used by SECHIBA
       !Config Def  = DEF 
       !Config Help = This option allows the user to impose a snow albedo.
       !Config        Default behaviour is to use the model of snow albedo
       !Config        deveoped by Chalita (1993).
       !
       fixed_snow_albedo = undef_sechiba
       CALL getin('CONDVEG_SNOWA', fixed_snow_albedo)
       
       l_first_condveg=.FALSE.
       !
       !  Allocate variables which have to
       ! 
       ALLOCATE (soilalb_dry(kjpindex,2),stat=ier)
        IF (ier.NE.0) THEN
          WRITE (numout,*) ' error in soilalb_dry allocation. We stop.We need kjpindex words = ',kjpindex
          STOP 'condveg_init'
        END IF

       ALLOCATE (soilalb_wet(kjpindex,2),stat=ier)
        IF (ier.NE.0) THEN
          WRITE (numout,*) ' error in soilalb_wet allocation. We stop.We need kjpindex words = ',kjpindex
          STOP 'condveg_init'
        END IF

       ALLOCATE (albedo_snow(kjpindex),stat=ier)
        IF (ier.NE.0) THEN
          WRITE (numout,*) ' error in albedo_snow allocation. We stop.We need kjpindex words = ',kjpindex
          STOP 'condveg_init'
        END IF

       ALLOCATE (albedo_glob(kjpindex),stat=ier)
        IF (ier.NE.0) THEN
          WRITE (numout,*) ' error in albedo_glob allocation. We stop.We need kjpindex words = ',kjpindex
          STOP 'condveg_init'
        END IF
       !
       !  Get the bare soil albedo
       !
       !
       var_name= 'soilalbedo_dry'
       CALL ioconf_setatt('UNITS', '-')
       CALL ioconf_setatt('LONG_NAME','Dry bare soil labedo')
       CALL restget (rest_id, var_name, kjpindex, 2, 1, kjit, .TRUE., soilalb_dry, "gather", kjpindex, index)

       var_name= 'soilalbedo_wet'
       CALL ioconf_setatt('UNITS', '-')
       CALL ioconf_setatt('LONG_NAME','Wet bare soil labedo')
       CALL restget (rest_id, var_name, kjpindex, 2, 1, kjit, .TRUE., soilalb_wet, "gather", kjpindex, index)
       !
       IF ( MINVAL(soilalb_wet) .EQ. MAXVAL(soilalb_wet) .AND. MAXVAL(soilalb_wet) .EQ. val_exp .OR.&
            & MINVAL(soilalb_dry) .EQ. MAXVAL(soilalb_dry) .AND. MAXVAL(soilalb_dry) .EQ. val_exp) THEN
          CALL condveg_soilalb(kjpindex, lalo, neighbours, resolution,soilalb_dry,soilalb_wet)
          WRITE(*,*) '---> val_exp ', val_exp
          WRITE(*,*) '---> ALBEDO_wet VIS:', MINVAL(soilalb_wet(:,ivis)), MAXVAL(soilalb_wet(:,ivis))
          WRITE(*,*) '---> ALBEDO_wet NIR:', MINVAL(soilalb_wet(:,inir)), MAXVAL(soilalb_wet(:,inir))
          WRITE(*,*) '---> ALBEDO_dry VIS:', MINVAL(soilalb_dry(:,ivis)), MAXVAL(soilalb_dry(:,ivis))
          WRITE(*,*) '---> ALBEDO_dry NIR:', MINVAL(soilalb_dry(:,inir)), MAXVAL(soilalb_dry(:,inir))
       ENDIF
       !
    ELSE 
       WRITE (numout,*) ' l_first_condveg false . we stop '
       STOP 'condveg_init'
    ENDIF
    !! test de commentaires

    IF (long_print) WRITE (numout,*) ' condveg_init done '

  END SUBROUTINE condveg_init
  !!
  !!
  SUBROUTINE condveg_clear  ()

      l_first_condveg=.TRUE.
       
       IF (ALLOCATED (soilalb_dry)) DEALLOCATE (soilalb_dry)
       IF (ALLOCATED(soilalb_wet))  DEALLOCATE (soilalb_wet)
       IF (ALLOCATED(albedo_snow))  DEALLOCATE (albedo_snow)
       IF (ALLOCATED(albedo_glob))  DEALLOCATE (albedo_glob)
       !
  END SUBROUTINE condveg_clear

  !! Algorithm:
  !! - initialisation of local arry
  !! - reads map for emissivity, albedo or roughness
  !!
  SUBROUTINE condveg_var_init  (ldrestart_read, kjpindex, veget, veget_max, frac_nobio, totfrac_nobio,&
       & hdry, zlev, height, deadleaf_cover, emis, albedo, z0, roughheight)

    ! interface description
    ! input scalar 
    INTEGER(long), INTENT(in)                       :: kjpindex         !! Domain size
    LOGICAL, INTENT(in)                             :: ldrestart_read   !! Logical for _restart_ file to read
    ! input fields
    REAL(stnd),DIMENSION (kjpindex,nvm), INTENT(in) :: veget            !! Fraction of vegetation types
    REAL(stnd),DIMENSION (kjpindex,nvm), INTENT(in) :: veget_max        !! Fraction of vegetation type
    REAL(stnd),DIMENSION (kjpindex,nnobio), INTENT(in) :: frac_nobio    !! Fraction of continental ice, lakes, ...
    REAL(stnd),DIMENSION (kjpindex), INTENT(in)     :: totfrac_nobio    !! Total fraction of continental ice+lakes+...
    REAL(stnd),DIMENSION (kjpindex), INTENT(in)     :: hdry             !! Dry soil heigth in meters
    REAL(stnd),DIMENSION (kjpindex), INTENT (in)    :: zlev             !! Height of first layer
    REAL(stnd),DIMENSION (kjpindex,nvm), INTENT(in) :: height           !! Vegetation Height (m)
    REAL(stnd),DIMENSION (kjpindex), INTENT(in)     :: deadleaf_cover   !! Fraction of soil covered by dead leaves
    ! output scalar
    ! output fields
    REAL(stnd),DIMENSION (kjpindex), INTENT (out)   :: emis             !! Emissivity
    REAL(stnd),DIMENSION (kjpindex,2), INTENT (out) :: albedo           !! Albedo, vis(1) and nir(2)
    REAL(stnd),DIMENSION (kjpindex), INTENT (out)   :: z0               !! Roughness
    REAL(stnd),DIMENSION (kjpindex), INTENT (out)   :: roughheight      !! Effective height for roughness
    !
    ! local declaration
    INTEGER(short)                                 :: ier               !! Error code
    INTEGER(short)                                 :: jv
    ! initialisation of variables
    !

    !
    !Config Key  = IMPOSE_AZE
    !Config Desc = Should thesurface parameters be prescribed
    !Config Def  = n
    !Config Help = This flag allows the user to impose the surface parameters
    !Config        (Albedo Roughness and Emissivity). It is espacially interesting for 0D
    !Config        simulations. On the globe it does not make too much sense as
    !Config        it imposes the same vegetation everywhere
    !
    impaze = .FALSE.
    CALL getin ('IMPOSE_AZE', impaze)

    !!
    !! calculs de emis
    !!

    IF ( impaze ) THEN
!
!Config Key  = CONDVEG_EMIS
!Config Desc = Emissivity of the surface for LW radiation
!Config Def  = 1.0
!Config If   = IMPOSE_AZE
!Config Help = The surface emissivity used for compution the LE emission
!Config        of the surface in a 0-dim version. Values range between 
!Config        0.97 and 1.. The GCM uses 0.98.
!
       emis_scal = un
       CALL getin('CONDVEG_EMIS', emis_scal)
       emis(:) = emis_scal

    ELSE
       !  Some day it will be moisture dependent
       emis_scal = un
       
       emis(:) = emis_scal
    ENDIF

    !!
    !! calculs de albedo
    !!
    !
    IF ( impaze ) THEN
      !
      !Config Key  = CONDVEG_ALBVIS
      !Config Desc = SW visible albedo for the surface
      !Config Def  = 0.25
      !Config If   = IMPOSE_AZE
      !Config Help = Surface albedo in visible wavelengths to be used 
      !Config        on the point if a 0-dim version of SECHIBA is used. 
      !Config        Look at the description of the forcing data for 
      !Config        the correct value.
      !
        albedo_scal(ivis) = 0.25_stnd
      CALL getin('CONDVEG_ALBVIS', albedo_scal(ivis))
       albedo(:,ivis) = albedo_scal(ivis)
      !
      !Config Key  = CONDVEG_ALBNIR
      !Config Desc = SW near infrared albedo for the surface
      !Config Def  = 0.25
      !Config If   = IMPOSE_AZE
      !Config Help = Surface albedo in near infrared wavelengths to be used 
      !Config        on the point if a 0-dim version of SECHIBA is used. 
      !Config        Look at the description of the forcing data for 
      !Config        the correct value.
      !
       albedo_scal(inir) = 0.25_stnd
      CALL getin('CONDVEG_ALBNIR', albedo_scal(inir))
       albedo(:,inir) = albedo_scal(inir)
    ELSE
       !
       CALL condveg_albcalc (kjpindex,deadleaf_cover,veget,hdry,albedo)
       !
    ENDIF
    !!
    !! calculs de z0
    !!
    !
    !Config Key  = Z0CDRAG_AVE
    !Config Desc = Average method for z0
    !Config Def  = y
    !Config Help = If this flag is set to true (y) then the neutral Cdrag
    !Config        is averaged instead of the log(z0). This should be
    !Config        the prefered option. We still wish to keep the other
    !Config        option so we can come back if needed. If this is
    !Config        desired then one should set Z0CDRAG_AVE=n
    z0cdrag_ave = .TRUE.
    CALL getin('Z0CDRAG_AVE', z0cdrag_ave)
    !!
    IF ( impaze ) THEN
      !
      !Config Key  = CONDVEG_Z0
      !Config Desc = Surface roughness (m)
      !Config Def  = 0.15
      !Config If   = IMPOSE_AZE
      !Config Help = Surface rougness to be used on the point if a 0-dim version
      !Config        of SECHIBA is used. Look at the description of the forcing  
      !Config        data for the correct value.
      !
      z0_scal = 0.15_stnd
      CALL getin('CONDVEG_Z0', z0_scal)
      z0(:) = z0_scal
      !
      !Config Key  = ROUGHHEIGHT
      !Config Desc = Height to be added to the height of the first level (m)
      !Config Def  = 0.0
      !Config If   = IMPOSE_AZE
      !Config Help = ORCHIDEE assumes that the atmospheric level height is counted
      !Config        from the zero wind level. Thus to take into account the roughness
      !Config        of tall vegetation we need to correct this by a certain fraction
      !Config        of the vegetation height. This is called the roughness height in
      !Config        ORCHIDEE talk.
      !
      roughheight_scal = zero
      CALL getin('ROUGHHEIGHT', roughheight_scal)
      roughheight(:) = roughheight_scal
      !
    ELSE
      !
       IF ( z0cdrag_ave ) THEN
          CALL condveg_z0cdrag(kjpindex, veget, veget_max, frac_nobio, totfrac_nobio, zlev, &
               &               height, z0, roughheight)
       ELSE
          CALL condveg_z0logz(kjpindex, veget, veget_max, frac_nobio, totfrac_nobio, height, &
               &              z0, roughheight)
       ENDIF
      !
    ENDIF
    !!
    !!
    IF (long_print) WRITE (numout,*) ' condveg_var_init done '

  END SUBROUTINE condveg_var_init
  !!
  !! Algorithm:
  !! - Simply update the emissivity, albedo and roughness fields
  !!
  SUBROUTINE condveg_var_update  (ldrestart_read, kjpindex, veget, veget_max, frac_nobio, totfrac_nobio, &
       & hdry, zlev, height, deadleaf_cover, emis, albedo, z0, roughheight)

    ! interface description
    ! input scalar 
    INTEGER(long), INTENT(in)                       :: kjpindex         !! Domain size
    LOGICAL, INTENT(in)                             :: ldrestart_read   !! Logical for _restart_ file to read
    ! input fields
    REAL(stnd),DIMENSION (kjpindex,nvm), INTENT(in) :: veget            !! Fraction of vegetation types
    REAL(stnd),DIMENSION (kjpindex,nvm), INTENT(in) :: veget_max        !! Fraction of vegetation type
    REAL(stnd),DIMENSION (kjpindex,nnobio), INTENT(in) :: frac_nobio    !! Fraction of continental ice, lakes, ...
    REAL(stnd),DIMENSION (kjpindex), INTENT(in)     :: totfrac_nobio    !! Total fraction of continental ice+lakes+...
    REAL(stnd),DIMENSION (kjpindex), INTENT(in)     :: hdry             !! Dry soil heigth in meters
    REAL(stnd),DIMENSION (kjpindex), INTENT (in)    :: zlev             !! Height of first layer
    REAL(stnd),DIMENSION (kjpindex,nvm), INTENT(in) :: height           !! Vegetation Height (m)
    REAL(stnd),DIMENSION (kjpindex), INTENT(in)     :: deadleaf_cover   !! Fraction of soil covered by dead leaves
    ! output scalar
    ! output fields
    REAL(stnd),DIMENSION (kjpindex), INTENT (out)   :: emis             !! Emissivity
    REAL(stnd),DIMENSION (kjpindex,2), INTENT (out) :: albedo           !! Albedo, vis(1) and nir(2)
    REAL(stnd),DIMENSION (kjpindex), INTENT (out)   :: z0               !! Roughness
    REAL(stnd),DIMENSION (kjpindex), INTENT (out)   :: roughheight      !! Effective height for roughness
    !
    ! local
    INTEGER(long)                                  :: ji, jv
    !!
    !! calculs de emis
    !!

    emis(:) = emis_scal
    
    !!
    !! calculs de albedo
    !!
    !
    IF ( impaze ) THEN
       !
       albedo(:,ivis) = albedo_scal(ivis)
       albedo(:,inir) = albedo_scal(inir)
       !
    ELSE
       !
       CALL condveg_albcalc (kjpindex,deadleaf_cover,veget,hdry,albedo)
       !
    ENDIF
    !
    !!
    !! Calculs de la rugosite
    !!
    
    IF ( impaze ) THEN

       DO ji = 1, kjpindex
         z0(ji) = z0_scal
         roughheight(ji) = roughheight_scal
      ENDDO

    ELSE
       !
       IF ( z0cdrag_ave ) THEN
          CALL condveg_z0cdrag (kjpindex, veget, veget_max, frac_nobio, totfrac_nobio, zlev, height, &
               &                z0, roughheight)
       ELSE
          CALL condveg_z0logz (kjpindex, veget, veget_max, frac_nobio, totfrac_nobio, height, &
               &               z0, roughheight)
       ENDIF
       !
    ENDIF

    IF (long_print) WRITE (numout,*) ' condveg_var_update done '

  END SUBROUTINE condveg_var_update

  !! Algorithm:
  !! - The snow albedo is updated by the snow within the mesh
  !! This is done as a function of snow mass, snow age and vegetation type
  !! The model is described in Chalita 1992
  !!
  SUBROUTINE condveg_snow  (ldrestart_read, kjpindex, veget, frac_nobio, totfrac_nobio, &
                            snow, snow_age, snow_nobio, snow_nobio_age, albedo, albedo_snow, albedo_glob)

    ! interface description
    ! input scalar 
    INTEGER(long), INTENT(in)                          :: kjpindex         !! Domain size
    LOGICAL, INTENT(in)                                :: ldrestart_read   !! Logical for _restart_ file to read
    ! input fields
    REAL(stnd),DIMENSION (kjpindex,nvm), INTENT (in)   :: veget            !! Fraction of vegetation types
    REAL(stnd),DIMENSION (kjpindex,nnobio), INTENT(in) :: frac_nobio       !! Fraction of continental ice, lakes, ...
    REAL(stnd),DIMENSION (kjpindex), INTENT(in)        :: totfrac_nobio    !! Total fraction of continental ice+lakes+ ...
    REAL(stnd),DIMENSION (kjpindex), INTENT(in)        :: snow             !! Snow mass [Kg/m^2] in vegetation
    REAL(stnd),DIMENSION (kjpindex,nnobio), INTENT(in) :: snow_nobio       !! Snow mass [Kg/m^2]on ice, lakes, ...
    REAL(stnd),DIMENSION (kjpindex), INTENT(in)        :: snow_age         !! Snow age
    REAL(stnd),DIMENSION (kjpindex,nnobio), INTENT(in) :: snow_nobio_age   !! Snow age on ice, lakes, ...
    ! output scalar
    ! output fields
    REAL(stnd),DIMENSION (kjpindex,2), INTENT (inout)  :: albedo           !! Albedo
    REAL(stnd),DIMENSION (kjpindex), INTENT (out)      :: albedo_snow      !! Albedo de la neige
    REAL(stnd),DIMENSION (kjpindex), INTENT (out)      :: albedo_glob      !! Mean albedo
    !
    ! local declaration
    INTEGER(short)                                  :: ji, jv, jb       !! Loop index
    REAL(stnd), DIMENSION(kjpindex)                 :: frac_snow_veg    !! Fraction of snow on vegetation
    REAL(stnd), DIMENSION(kjpindex,nnobio)          :: frac_snow_nobio  !! Fraction of snow on ice, lakes, ...
    REAL(stnd), DIMENSION(kjpindex)                 :: snowa_veg        !! Total albedo of snow covered area on vegetation
    REAL(stnd), DIMENSION(kjpindex,nnobio)          :: snowa_nobio      !! albedo of snow covered area on ice, lakes, ...
    REAL(stnd), DIMENSION(kjpindex)                 :: fraction_veg     !! total vegetation fraction
    REAL(stnd), DIMENSION(kjpindex)                 :: agefunc_veg      !! age dependency of snow albedo on vegetation
    REAL(stnd), DIMENSION(kjpindex,nnobio)          :: agefunc_nobio    !! age dependency of snow albedo on ice, lakes, ..;
    REAL(stnd)                                      :: alb_nobio
    !

    DO ji = 1, kjpindex
     albedo_snow(ji) = zero
     albedo_glob(ji) = zero
    ENDDO

    IF (ABS(fixed_snow_albedo - undef_sechiba) .GT. EPSILON(undef_sechiba)) THEN
      snowa_veg(:) = fixed_snow_albedo
      snowa_nobio(:,:) = fixed_snow_albedo
    ELSE
      !
      ! calculate first age dependence
      !
      DO ji = 1, kjpindex
        agefunc_veg(ji) = EXP(-snow_age(ji)/tcst_snowa)
      ENDDO
      !
      DO jv = 1, nnobio
        DO ji = 1, kjpindex
          agefunc_nobio(ji,jv) = EXP(-snow_nobio_age(ji,jv)/tcst_snowa)
        ENDDO
      ENDDO
      !
      ! snow albedo on vegetated surfaces
      !
      fraction_veg(:) = 1. - totfrac_nobio(:)
      snowa_veg(:) = 0.
      DO jv = 1, nvm
        DO ji = 1, kjpindex
          IF ( fraction_veg(ji) .GT. 0. ) THEN
            snowa_veg(ji) = snowa_veg(ji) + &
              veget(ji,jv)/fraction_veg(ji) * ( snowa_ini(jv)+snowa_dec(jv)*agefunc_veg(ji) )
          ENDIF
        ENDDO
      ENDDO
      !
      ! snow albedo on other surfaces
      !
      DO jv = 1, nnobio
        DO ji = 1, kjpindex
          snowa_nobio(ji,jv) = ( snowa_ini(1) + snowa_dec(1) * agefunc_nobio(ji,jv) ) 
        ENDDO
      ENDDO
    ENDIF

    frac_snow_veg(:) = MIN(MAX(snow(:),zero)/(MAX(snow(:),zero)+snowcri_alb),un)
    DO jv = 1, nnobio
      frac_snow_nobio(:,jv) = MIN(MAX(snow_nobio(:,jv),zero)/(MAX(snow_nobio(:,jv),zero)+snowcri_alb),un)
    ENDDO

    DO jb = 1, 2
      !
      albedo(:,jb) = ( fraction_veg(:) ) * &
                         ( (un-frac_snow_veg(:)) * albedo(:,jb) + &
                           ( frac_snow_veg(:)  ) * snowa_veg(:)    )
      albedo_snow(:) =  albedo_snow(:) + (fraction_veg(:)) * (frac_snow_veg(:)) * snowa_veg(:)
      !
      DO jv = 1, nnobio
        ! 
        IF ( jv .EQ. iice ) THEN
          alb_nobio = alb_ice(jb)
        ELSE
          WRITE(*,*) 'jv=',jv
          STOP 'DO NOT KNOW ALBEDO OF THIS SURFACE TYPE'
        ENDIF
        !
        albedo(:,jb) = albedo(:,jb) + &
                       ( frac_nobio(:,jv) ) * &
                         ( (un-frac_snow_nobio(:,jv)) * alb_nobio + &
                           ( frac_snow_nobio(:,jv)  ) * snowa_nobio(:,jv)   )
        albedo_snow(:) = albedo_snow(:) + &
                         ( frac_nobio(:,jv) ) * ( frac_snow_nobio(:,jv) ) * &
                           snowa_nobio(:,jv)
        albedo_glob(:) = albedo_glob(:) + albedo(:,jb)
        !
      ENDDO
      !
    END DO
    !
    DO ji = 1, kjpindex
      albedo_snow(ji) = (albedo_snow(ji))/2.
      albedo_glob(ji) = (albedo_glob(ji))/2.
    ENDDO
    

    IF (long_print) WRITE (numout,*) ' condveg_snow done '

  END SUBROUTINE condveg_snow
!
!
!
SUBROUTINE condveg_soilalb(nbpt, lalo, neighbours, resolution,soilalb_dry,soilalb_wet)
   !
  !
  !   This subroutine should read the soil color maps from the Henderson-Sellers & Wilson database. This data
  !   is then interpolated to the models resolution and transformed into dry and wet albedos. We have chosen to
  !  do both these operations at the same time as one can average the albedo but not the color types.
  !
  !  We make the assumption in this code that the grid of the data is regular and that it covers the globe. 
  !  For the model grid we base the calculation of the borders of the grid on the resolution.
  !
  !
  !  0.1 INPUT
  !
  INTEGER(long), INTENT(in)    :: nbpt                ! Number of points for which the data needs to be interpolated
  REAL(stnd), INTENT(in)       :: lalo(nbpt,2)        ! Vector of latitude and longitudes (beware of the order !)
  INTEGER(long), INTENT(in)    :: neighbours(nbpt,8)  ! Vector of neighbours for each grid point (1=N, 2=E, 3=S, 4=W)
  REAL(stnd), INTENT(in)       :: resolution(nbpt,2)  ! The size in km of each grid-box in X and Y
  REAL(stnd), INTENT(inout)    :: soilalb_dry(nbpt,2) ! albedo for the dry bare soil
  REAL(stnd), INTENT(inout)    :: soilalb_wet(nbpt,2) ! albedo for the wet bare soil

  !
  !
  !  0.3 LOCAL
  !
  INTEGER(long), PARAMETER :: nbvmax=200
  REAL(stnd), PARAMETER                          :: R_Earth = 6378000.
  !
  CHARACTER*80 :: filename
  INTEGER(long) :: iml, jml, lml, tml, fid, ib, ip, jp, fopt, ilf, lastjp, nbexp
  REAL(stnd) :: lev(1), date, dt, coslat, pi
  INTEGER(long) :: itau(1)
  REAL(stnd), ALLOCATABLE, DIMENSION(:,:) :: lat_rel, lon_rel, lat_ful, lon_ful, soilcol
  REAL(stnd), ALLOCATABLE, DIMENSION(:,:) :: loup_rel, lolow_rel, laup_rel, lalow_rel
  REAL(stnd) :: lon_up, lon_low, lat_up, lat_low
  INTEGER(long) :: solc(nbvmax)
  REAL(stnd) :: area(nbvmax), ax, ay, sgn
  REAL(stnd) :: lonrel, louprel, lolowrel
  !
  !   The correspondance table for the soil color numbers and their albedo
  !
  INTEGER(long), PARAMETER :: classnb = 9
  !
!  REAL(stnd), DIMENSION(classnb) :: vis_dry = (/0.10, 0.12, 0.14, 0.16, 0.18, 0.20, 0.22, 0.24, 0.27/)
!  REAL(stnd), DIMENSION(classnb) :: nir_dry = (/0.20, 0.24, 0.28, 0.32, 0.36, 0.40, 0.44, 0.48, 0.55/)  
!  REAL(stnd), DIMENSION(classnb) :: vis_wet = (/0.05, 0.06, 0.07, 0.08, 0.09, 0.10, 0.11, 0.12, 0.15/)  
!  REAL(stnd), DIMENSION(classnb) :: nir_wet = (/0.10, 0.12, 0.14, 0.16, 0.18, 0.20, 0.22, 0.24, 0.31/)
  REAL(stnd), DIMENSION(classnb) :: vis_dry = (/0.24, 0.22, 0.20, 0.18, 0.16, 0.14, 0.12, 0.10, 0.27/)
  REAL(stnd), DIMENSION(classnb) :: nir_dry = (/0.48, 0.44, 0.40, 0.36, 0.32, 0.28, 0.24, 0.20, 0.55/)  
  REAL(stnd), DIMENSION(classnb) :: vis_wet = (/0.12, 0.11, 0.10, 0.09, 0.08, 0.07, 0.06, 0.05, 0.15/)  
  REAL(stnd), DIMENSION(classnb) :: nir_wet = (/0.24, 0.22, 0.20, 0.18, 0.16, 0.14, 0.12, 0.10, 0.31/)
  !
  pi = 4. * ATAN(1.)
  !
  !  Needs to be a configurable variable
  !
  !
  !Config Key  = SOILALB_FILE
  !Config Desc = Name of file from which the bare soil albedo
  !Config Def  = ../surfmap/soils_param.nc
  !Config If   = !IMPOSE_AZE
  !Config Help = The name of the file to be opened to read the soil types from 
  !Config        which we derive then the bare soil albedos. This file is 1x1 
  !Config        deg and based on the soil colors defined by Wilson and Henderson-Seller.
  !
  filename = '../surfmap/soils_param.nc'
  CALL getin('SOILALB_FILE',filename)
  !
  CALL flininfo(filename,iml, jml, lml, tml, fid)
  !
  !
  ALLOCATE (lat_rel(iml,jml))
  ALLOCATE (lon_rel(iml,jml))
  ALLOCATE (laup_rel(iml,jml))
  ALLOCATE (loup_rel(iml,jml))
  ALLOCATE (lalow_rel(iml,jml))
  ALLOCATE (lolow_rel(iml,jml))
  ALLOCATE (lat_ful(iml+2,jml+2))
  ALLOCATE (lon_ful(iml+2,jml+2))
  ALLOCATE (soilcol(iml,jml))
  !
  CALL flinopen(filename, .FALSE., iml, jml, lml, lon_rel, lat_rel, lev, tml, itau, date, dt, fid)
  !
  CALL flinget(fid, 'soilcolor', iml, jml, lml, tml, 1, 1, soilcol)
  !
  CALL flinclo(fid)
  !
  nbexp = 0
  !
  !    Duplicate the border assuming we have a global grid going from west to east
  !
  lon_ful(2:iml+1,2:jml+1) = lon_rel(1:iml,1:jml)
  lat_ful(2:iml+1,2:jml+1) = lat_rel(1:iml,1:jml)
  !
  IF ( lon_rel(iml,1) .LT. lon_ful(2,2)) THEN
      lon_ful(1,2:jml+1) = lon_rel(iml,1:jml)
      lat_ful(1,2:jml+1) = lat_rel(iml,1:jml)
  ELSE
      lon_ful(1,2:jml+1) = lon_rel(iml,1:jml)-360
      lat_ful(1,2:jml+1) = lat_rel(iml,1:jml)
  ENDIF

  IF ( lon_rel(1,1) .GT. lon_ful(iml+1,2)) THEN
      lon_ful(iml+2,2:jml+1) = lon_rel(1,1:jml)
      lat_ful(iml+2,2:jml+1) = lat_rel(1,1:jml)
  ELSE
      lon_ful(iml+2,2:jml+1) = lon_rel(1,1:jml)+360
      lat_ful(iml+2,2:jml+1) = lat_rel(1,1:jml)
  ENDIF
  !
  sgn = lat_rel(1,1)/ABS(lat_rel(1,1))
  lat_ful(2:iml+1,1) = sgn*180 - lat_rel(1:iml,1)
  sgn = lat_rel(1,jml)/ABS(lat_rel(1,jml))
  lat_ful(2:iml+1,jml+2) = sgn*180 - lat_rel(1:iml,jml)
  lat_ful(1,1) = lat_ful(iml+1,1)
  lat_ful(iml+2,1) = lat_ful(2,1)
  lat_ful(1,jml+2) = lat_ful(iml+1,jml+2)
  lat_ful(iml+2,jml+2) = lat_ful(2,jml+2)
  !
  ! Add the longitude lines to the top and bottom
  !
  lon_ful(:,1) = lon_ful(:,2) 
  lon_ful(:,jml+2) = lon_ful(:,jml+1) 
  !
  !  Get the upper and lower limits of each grid box
  !
  DO ip=1,iml
    DO jp=1,jml
      loup_rel(ip,jp) =MAX(0.5*(lon_ful(ip,jp+1)+lon_ful(ip+1,jp+1)), 0.5*(lon_ful(ip+1,jp+1)+lon_ful(ip+2,jp+1)))
      lolow_rel(ip,jp) =MIN(0.5*(lon_ful(ip,jp+1)+lon_ful(ip+1,jp+1)), 0.5*(lon_ful(ip+1,jp+1)+lon_ful(ip+2,jp+1)))
      laup_rel(ip,jp) =MAX(0.5*(lat_ful(ip+1,jp)+lat_ful(ip+1,jp+1)), 0.5*(lat_ful(ip+1,jp+1)+lat_ful(ip+1,jp+2)))
      lalow_rel(ip,jp) =MIN(0.5*(lat_ful(ip+1,jp)+lat_ful(ip+1,jp+1)), 0.5*(lat_ful(ip+1,jp+1)+lat_ful(ip+1,jp+2)))
    ENDDO
  ENDDO
  !
  !   Now we take each grid point and find out which values from the forcing we need to average
  !
  DO ib =1, nbpt
     !
     !  We find the 4 limits of the grid-box. As we transform the resolution of the model
     !  into longitudes and latitudes we do not have the problem of periodicity.
     ! coslat is a help variable here !
     !
     coslat = MAX(COS(lalo(ib,1) * pi/180. ), 0.001 )*pi/180. * R_Earth
     !
     lon_up = lalo(ib,2)+ resolution(ib,1)/(2.0*coslat) 
     lon_low =lalo(ib,2) - resolution(ib,1)/(2.0*coslat) 
     !
     coslat = pi/180. * R_Earth
     !
     lat_up =lalo(ib,1)+resolution(ib,2)/(2.0*coslat) 
     lat_low =lalo(ib,1)-resolution(ib,2)/(2.0*coslat) 
     !
     !
     !  Find the grid boxes from the data that go into the model's boxes
     !  We still work as if we had a regular grid ! Well it needs to be localy regular so
     !  so that the longitude at the latitude of the last found point is close to the one of the next point.
     !
     fopt = 0
     lastjp = 1
     DO ip=1,iml
        !
        !  Either the center of the data grid point is in the interval of the model grid or
        !  the East and West limits of the data grid point are on either sides of the border of
        !  the data grid.
        !
        !
        !  To do that correctly we have to check if the grid box sits on the date-line.
        !
        IF ( lon_low < -180.0 ) THEN
           lonrel = MOD( lon_rel(ip,lastjp) - 360.0, 360.0)
           lolowrel = MOD( lolow_rel(ip,lastjp) - 360.0, 360.0)
           louprel = MOD( loup_rel(ip,lastjp) - 360.0, 360.0)
           !
        ELSE IF ( lon_up > 180.0 ) THEN
           lonrel = MOD( 360. - lon_rel(ip,lastjp), 360.0)
           lolowrel = MOD( 360. - lolow_rel(ip,lastjp), 360.0)
           louprel = MOD( 360. - loup_rel(ip,lastjp), 360.0)
        ELSE
           lonrel = lon_rel(ip,lastjp)
           lolowrel = lolow_rel(ip,lastjp)
           louprel = loup_rel(ip,lastjp)
       ENDIF
       !
       !
       !
       IF ( lonrel > lon_low .AND. lonrel < lon_up .OR. &
            & lolowrel < lon_low .AND.  louprel > lon_low .OR. &
            & lolowrel < lon_up  .AND.  louprel > lon_up ) THEN
          !  
          DO jp = 1, jml
            !
            ! Now that we have the longitude let us find the latitude
            !
            IF ( lat_rel(ip,jp) > lat_low .AND. lat_rel(ip,jp) < lat_up .OR. &
               & lalow_rel(ip,jp) < lat_low .AND. laup_rel(ip,jp) > lat_low .OR.&
               & lalow_rel(ip,jp) < lat_up .AND. laup_rel(ip,jp) > lat_up) THEN
                !
                lastjp = jp
                !
                IF (soilcol(ip,jp) .GT. min_sechiba) THEN
                    !
                    fopt = fopt + 1
                    IF ( fopt .GT. nbvmax) THEN
                        WRITE(*,*) 'Please increase nbvmax in subroutine albedo_interpol', ib
                        STOP
                    ELSE
                        !
                        ! If we sit on the date line we need to do the same transformations as above.
                        ! 
                        IF ( lon_low < -180.0 ) THEN
                           lolowrel = MOD( lolow_rel(ip,jp) - 360.0, 360.0)
                           louprel = MOD( loup_rel(ip,jp) - 360.0, 360.0)
                           !
                        ELSE IF ( lon_up > 180.0 ) THEN
                           lolowrel = MOD( 360. - lolow_rel(ip,jp), 360.0)
                           louprel = MOD( 360. - loup_rel(ip,jp), 360.0)
                        ELSE
                           lolowrel = lolow_rel(ip,jp)
                           louprel = loup_rel(ip,jp)
                        ENDIF
                        !
                        ! Get the area of the fine grid in the model grid
                        !
                        coslat = MAX( COS( lat_rel(ip,jp) * pi/180. ), 0.001 )
                        ax = (MIN(lon_up,louprel)-MAX(lon_low, lolowrel))*pi/180. * R_Earth * coslat
                        ay = (MIN(lat_up, laup_rel(ip,jp))-MAX(lat_low,lalow_rel(ip,jp)))*pi/180. * R_Earth
                        area(fopt) = ax*ay
                        solc(fopt) = NINT(soilcol(ip,jp))
                    ENDIF
                ENDIF
                !
            ENDIF
            !
          ENDDO
          !
      ENDIF
      !
    ENDDO
    !
    !    Check that we found some points
    !
    soilalb_dry(ib,:) = 0.0
    soilalb_wet(ib,:) = 0.0
    !
    IF ( fopt .EQ. 0) THEN
       nbexp = nbexp + 1
       soilalb_dry(ib,ivis) = (SUM(vis_dry)/classnb + SUM(vis_wet)/classnb)/deux
       soilalb_dry(ib,inir) = (SUM(nir_dry)/classnb + SUM(nir_wet)/classnb)/deux
       soilalb_wet(ib,ivis) = (SUM(vis_dry)/classnb + SUM(vis_wet)/classnb)/deux
       soilalb_wet(ib,inir) = (SUM(nir_dry)/classnb + SUM(nir_wet)/classnb)/deux
    ELSE
        sgn = zero
        !
        !   Compute the average bare soil albedo parameters
        !
        DO ilf = 1,fopt
           IF ( solc(ilf) .LE. classnb) THEN
              soilalb_dry(ib,ivis) = soilalb_dry(ib,ivis) + vis_dry(solc(ilf)) * area(ilf)
              soilalb_dry(ib,inir) = soilalb_dry(ib,inir) + nir_dry(solc(ilf)) * area(ilf)
              soilalb_wet(ib,ivis) = soilalb_wet(ib,ivis) + vis_wet(solc(ilf)) * area(ilf)
              soilalb_wet(ib,inir) = soilalb_wet(ib,inir) + nir_wet(solc(ilf)) * area(ilf)
              sgn = sgn + area(ilf)
           ELSE
              WRITE(*,*) 'The file contains a soil color class which is incompatible with this program'
              STOP
           ENDIF
           !
        ENDDO
        !
        ! Normalize the surface
        !
        IF ( sgn .LT. min_sechiba) THEN
           nbexp = nbexp + 1
           soilalb_dry(ib,ivis) = (SUM(vis_dry)/classnb + SUM(vis_wet)/classnb)/deux
           soilalb_dry(ib,inir) = (SUM(nir_dry)/classnb + SUM(nir_wet)/classnb)/deux
           soilalb_wet(ib,ivis) = (SUM(vis_dry)/classnb + SUM(vis_wet)/classnb)/deux
           soilalb_wet(ib,inir) = (SUM(nir_dry)/classnb + SUM(nir_wet)/classnb)/deux
        ELSE
           soilalb_dry(ib,ivis) = soilalb_dry(ib,ivis)/sgn
           soilalb_dry(ib,inir) = soilalb_dry(ib,inir)/sgn
           soilalb_wet(ib,ivis) = soilalb_wet(ib,ivis)/sgn
           soilalb_wet(ib,inir) = soilalb_wet(ib,inir)/sgn
        ENDIF
        !
    ENDIF
    !
  ENDDO
  !
  IF ( nbexp .GT. 0 ) THEN
     WRITE(*,*) 'CONDVEG_soilalb : The interpolation of the bare soil albedo had ', nbexp
     WRITE(*,*) 'CONDVEG_soilalb : points without data. This are either coastal points or'
     WRITE(*,*) 'CONDVEG_soilalb : ice covered land.'
     WRITE(*,*) 'CONDVEG_soilalb : The problem was solved by using the average of all soils'
     WRITE(*,*) 'CONDVEG_soilalb : in dry and wet conditions'
  ENDIF
  !
  DEALLOCATE (lat_rel)
  DEALLOCATE (lon_rel)
  DEALLOCATE (laup_rel)
  DEALLOCATE (loup_rel)
  DEALLOCATE (lalow_rel)
  DEALLOCATE (lolow_rel)
  DEALLOCATE (lat_ful)
  DEALLOCATE (lon_ful)
  DEALLOCATE (soilcol)
  !
  !
  RETURN
  !
END SUBROUTINE condveg_soilalb
!
!
!
SUBROUTINE condveg_z0logz (kjpindex, veget, veget_max, frac_nobio, totfrac_nobio, height, &
     &                     z0, roughheight)

  !
  ! 0. Declarations
  !

  ! 0.1 Input
  INTEGER(long), INTENT(in)                       :: kjpindex
  REAL(stnd), DIMENSION(kjpindex,nvm), INTENT(in) :: veget
  REAL(stnd), DIMENSION(kjpindex,nvm), INTENT(in) :: veget_max
  REAL(stnd), DIMENSION(kjpindex,nnobio), INTENT(in) :: frac_nobio
  REAL(stnd), DIMENSION(kjpindex), INTENT(in)     :: totfrac_nobio
  REAL(stnd), DIMENSION(kjpindex,nvm), INTENT(in) :: height

  ! 0.2 Output
  REAL(stnd), DIMENSION(kjpindex), INTENT(out)    :: z0
  REAL(stnd), DIMENSION(kjpindex), INTENT(out)    :: roughheight

  ! 0.3 Local
  INTEGER(short)                                 :: jv
  REAL(stnd), DIMENSION(kjpindex)                 :: sumveg, ave_height
  REAL(stnd), DIMENSION(kjpindex)                 :: d_veg, zhdispl
  REAL(stnd)                                      :: z0_nobio

  z0(:) = veget(:,1) * LOG(z0_bare)
  sumveg(:) = veget(:,1)
  ave_height(:) = zero
  !
  DO jv = 2, nvm
     !
     IF ( is_tree(jv) ) THEN
       ! tree trunks influence the atmosphere even when there are no leaves
       d_veg(:) = veget_max(:,jv)
     ELSE
       ! grasses only have an influence if they are really there!
       d_veg(:) = veget(:,jv)
     ENDIF
     !
     z0(:) = z0(:) + d_veg(:) * &
                     LOG( MAX(height(:,jv)*z0_over_height,z0_bare) )
     sumveg(:) = sumveg(:) + d_veg(:)
     !
     ave_height(:) = ave_height(:) + veget_max(:,jv)*height(:,jv)
     !
  ENDDO
  !
  WHERE ( sumveg(:) .GT. 0.0 ) z0(:) = z0(:) / sumveg(:)
  !
  z0(:) = (un - totfrac_nobio(:)) * z0(:)
  !
  DO jv = 1, nnobio
    !
    IF ( jv .EQ. iice ) THEN
      z0_nobio = z0_ice
    ELSE
      WRITE(*,*) 'jv=',jv
      STOP 'DO NOT KNOW ROUGHNESS OF THIS SURFACE TYPE'
    ENDIF
    !
    z0(:) = z0(:) + frac_nobio(:,jv) * LOG(z0_nobio)
    !
  ENDDO
  !
  z0(:) = EXP( z0(:) )
  !
  ! Temporarily we compute the zero plane displacement height
  !
  zhdispl(:) = ave_height(:) * height_displacement
  !
  ! In order to get a variable independent of the height of the
  ! vegetation we compute what we call the effective roughness height.
  ! This is the height over which the roughness acts. It combines the
  ! zero plane displacement height and the vegetation height.
  !
  roughheight(:) = ave_height(:) - zhdispl(:)
  !
END SUBROUTINE condveg_z0logz
!
!
!
SUBROUTINE condveg_z0cdrag (kjpindex,veget,veget_max,frac_nobio,totfrac_nobio,zlev, height, &
     &                      z0, roughheight)

  !
  ! 0. Declarations
  !

  ! 0.1 Input
  INTEGER(long), INTENT(in)                       :: kjpindex
  REAL(stnd), DIMENSION(kjpindex,nvm), INTENT(in) :: veget
  REAL(stnd), DIMENSION(kjpindex,nvm), INTENT(in) :: veget_max
  REAL(stnd), DIMENSION(kjpindex,nnobio), INTENT(in) :: frac_nobio
  REAL(stnd), DIMENSION(kjpindex), INTENT(in)     :: totfrac_nobio
  REAL(stnd),DIMENSION (kjpindex), INTENT (in)    :: zlev             !! Height of first layer
  REAL(stnd), DIMENSION(kjpindex,nvm), INTENT(in) :: height    

  ! 0.2 Output
  REAL(stnd), DIMENSION(kjpindex), INTENT(out)    :: z0
  REAL(stnd), DIMENSION(kjpindex), INTENT(out)    :: roughheight

  ! 0.3 Local
  INTEGER(short)                                 :: jv
  REAL(stnd), DIMENSION(kjpindex)                 :: sumveg, ztmp, ave_height
  REAL(stnd), DIMENSION(kjpindex)                 :: d_veg, zhdispl
  REAL(stnd)                                      :: z0_nobio
  !
  ! The grid average z0 is computed by averaging the neutral drag coefficients.
  ! This is pretty straight forward except for the reference level which needs
  ! to be chosen.
  !
  ! We need a reference lever high enough above the canopy else we get into
  ! singularities of the LOG.
  !
  ztmp(:) = MAX(10., zlev(:))
  !
  z0(:) = veget(:,1) * (ct_karman/LOG(ztmp(:)/z0_bare))**2
  sumveg(:) = veget(:,1)
  ave_height(:) = zero
  !
  DO jv = 2, nvm
     !
     IF ( is_tree(jv) ) THEN
       ! tree trunks influence the atmosphere even when there are no leaves
       d_veg(:) = veget_max(:,jv)
     ELSE
       ! grasses only have an influence if they are really there!
       d_veg(:) = veget(:,jv)
     ENDIF
     !
     z0(:) = z0(:) + d_veg(:) * (ct_karman/LOG(ztmp(:)/MAX(height(:,jv)*z0_over_height,z0_bare)))**2
     sumveg(:) = sumveg(:) + d_veg(:)
     !
     ave_height(:) = ave_height(:) + veget_max(:,jv)*height(:,jv)
     !
  ENDDO
  !
  WHERE ( sumveg(:) .GT. 0.0 ) z0(:) = z0(:) / sumveg(:)
  !
  z0(:) = (un - totfrac_nobio(:)) * z0(:)
  !
  DO jv = 1, nnobio
    !
    IF ( jv .EQ. iice ) THEN
      z0_nobio = z0_ice
    ELSE
      WRITE(*,*) 'jv=',jv
      STOP 'DO NOT KNOW ROUGHNESS OF THIS SURFACE TYPE'
    ENDIF
    !
    z0(:) = z0(:) + frac_nobio(:,jv) * (ct_karman/LOG(ztmp(:)/z0_nobio))**2
    !
  ENDDO
  !
  z0(:) = ztmp(:) / EXP(ct_karman/SQRT(z0(:)))
  !
  ! Temporarily we compute the zero plane displacement height
  !
  zhdispl(:) = ave_height(:) * height_displacement
  !
  ! In order to get a variable independent of the height of the
  ! vegetation we compute what we call the effective roughness height.
  ! This is the height over which the roughness acts. It combines the
  ! zero plane displacement height and the vegetation height.
  !
  roughheight(:) = ave_height(:) - zhdispl(:)
  !
END SUBROUTINE condveg_z0cdrag
!
!
!
SUBROUTINE condveg_albcalc (kjpindex,deadleaf_cover,veget,hdry,albedo)

  !
  ! 0. Declarations
  !

  ! 0.1 Input
  INTEGER(long), INTENT(in)                       :: kjpindex
  REAL(stnd), DIMENSION(kjpindex), INTENT(in)     :: deadleaf_cover
  REAL(stnd), DIMENSION(kjpindex,nvm), INTENT(in) :: veget
  REAL(stnd), DIMENSION(kjpindex), INTENT(in)     :: hdry

  ! 0.2 Output
  REAL(stnd),DIMENSION (kjpindex,2), INTENT (out) :: albedo  !! Albedo, vis(1) and nir(2)
  ! 0.3 Local
  REAL(stnd),DIMENSION (kjpindex)                 :: fdry, alb_bare
  REAL(stnd),DIMENSION (nvm,2)                    :: alb_leaf_tmp
  INTEGER(long)                                   :: ks, jv

  ! The factor which goes from wet to dry soil (dry when hdry = 0.1 m)
  fdry(:) = MIN(MAX(hdry(:),0.)*10._stnd, un)
  alb_leaf_tmp(:,1) = alb_leaf(1:nvm)
  alb_leaf_tmp(:,2) = alb_leaf(nvm+1:2*nvm)
  !
  DO ks = 1, 2
     !
     alb_bare(:) = soilalb_wet(:,ks) + fdry(:) * (soilalb_dry(:,ks) -  soilalb_wet(:,ks))
     !
     albedo(:,ks) = veget(:,1) * ( (1.-deadleaf_cover(:))*alb_bare(:) + &
                                   deadleaf_cover(:)*alb_deadleaf(ks)    )
     ! vegetation
     DO jv = 2, nvm
       albedo(:,ks) = albedo(:,ks) + veget(:,jv)*alb_leaf_tmp(jv,ks)
     ENDDO
     !
  ENDDO

END SUBROUTINE condveg_albcalc

END MODULE condveg
